@@ -2,11 +2,11 @@
 
 JAVA_HOME=/usr/local/java
 CLASSPATH=../build:.
-PROPERTIES=../../user10.prp
+PROPERTIES=../../user1.prp
 RUN="${JAVA_HOME}/bin/java -cp ${CLASSPATH} -Djcifs.properties=${PROPERTIES}"
 
 SERVER=dc1.w.net
-SHARE=tmp
+SHARE=target1
 WRITE_DIR=test/
 SRC_DIR=test/Junk
 FILE1=test/Junk/10883563.doc
