@@ -145,6 +145,8 @@ class NameServiceClient implements Runnable {
         // be ignored; see tryClose comment.
         if( socket == null ) {
             socket = new DatagramSocket( lport, laddr );
+jcifs.Config.socketCount++;
+log.println(jcifs.Config.socketCount + ": new Socket: " + socket);
             thread = new Thread( this, "JCIFS-NameServiceClient" );
             thread.setDaemon( true );
             thread.start();
@@ -162,7 +164,10 @@ class NameServiceClient implements Runnable {
              */
 
             if( socket != null ) {
+String s = socket.toString();
                 socket.close();
+jcifs.Config.socketCount--;
+log.println(jcifs.Config.socketCount + ": socket closed: " + s);
                 socket = null;
             }
             thread = null;
@@ -237,10 +242,21 @@ class NameServiceClient implements Runnable {
                         }
                     }
 
-                    response.wait( timeout );
+                    long start = System.currentTimeMillis();
+                    while (timeout > 0) {
+                        response.wait( timeout );
 
-                    if (response.received)
-                        return;
+                        /* JetDirect printer can respond to regular broadcast query
+                         * with node status so we need to check to make sure that
+                         * the record type matches the question type and if not,
+                         * loop around and try again.
+                         */
+                        if (response.received && request.questionType == response.recordType)
+                            return;
+
+                        response.received = false;
+                        timeout -= System.currentTimeMillis() - start;
+                    }
 
                 } catch( InterruptedException ie ) {
                 } finally {
