@@ -20,19 +20,23 @@ package jcifs.smb;
 
 public class DfsReferral extends SmbException {
 
-    public String path;     // Path relative to tree from which this referral was thrown
-    public String node;     // Server and share
+    public int pathConsumed;
+    public long ttl;
     public String server;   // Server
     public String share;    // Share
-    public String nodepath; // Path relative to tree
+    public String link;
+    public String path;     // Path relative to tree from which this referral was thrown
     public boolean resolveHashes;
+    public long expiration;
 
     public String toString() {
-        return "DfsReferral[path=" + path +
-            ",node=" + node +
+        return "DfsReferral[pathConsumed=" + pathConsumed +
             ",server=" + server +
             ",share=" + share +
-            ",nodepath=" + nodepath +
+            ",link=" + link +
+            ",path=" + path +
+            ",ttl=" + ttl +
+            ",expiration=" + expiration +
             ",resolveHashes=" + resolveHashes + "]";
     }
 }
