@@ -183,6 +183,10 @@ public final class NtlmPasswordAuthentication implements Principal, Serializable
         domain = username = password = null;
 
         if( userInfo != null ) {
+            try {
+                userInfo = unescape( userInfo );
+            } catch( UnsupportedEncodingException uee ) {
+            }
             int i, u, end;
             char c;
 
@@ -250,6 +254,7 @@ public final class NtlmPasswordAuthentication implements Principal, Serializable
     public String getDomain() {
         return domain;
     }
+
 /**
  * Returns the username.
  */
@@ -437,5 +442,43 @@ public final class NtlmPasswordAuthentication implements Principal, Serializable
     public String toString() {
         return getName();
     }
+
+    static String unescape( String str ) throws NumberFormatException, UnsupportedEncodingException {
+        char ch;
+        int i, j, state, len;
+        char[] out;
+        byte[] b = new byte[1];
+
+        if( str == null ) {
+            return null;
+        }
+
+        len = str.length();
+        out = new char[len];
+        state = 0;
+        for( i = j = 0; i < len; i++ ) {
+            switch( state ) {
+                case 0:
+                    ch = str.charAt( i );
+                    if( ch == '%' ) {
+                        state = 1;
+                    } else {
+                        out[j++] = ch;
+                    }
+                    break;
+                case 1:
+                    /* Get ASCII hex value and convert to platform dependant
+                     * encoding like EBCDIC perhaps
+                     */
+                    b[0] = (byte)(Integer.parseInt( str.substring( i, i + 2 ), 16 ) & 0xFF);
+                    out[j++] = (new String( b, 0, 1, "ASCII" )).charAt( 0 );
+                    i++;
+                    state = 0;
+            }
+        }
+
+        return new String( out, 0, j );
+    }
+
 }
 
