@@ -94,8 +94,8 @@ int retry = 1;
 
 do {
             if (dc_list_expiration < now) {
-                dc_list_expiration = now + CACHE_POLICY * 1000L;
                 NbtAddress[] list = NbtAddress.getAllByName( DOMAIN, 0x1C, null, null );
+                dc_list_expiration = now + CACHE_POLICY * 1000L;
                 if (list != null && list.length > 0) {
                     dc_list = list;
                 } else { /* keep using the old list */
@@ -175,7 +175,7 @@ do {
     }
 
     private int uid;
-    private Vector trees;
+    Vector trees;
     private boolean sessionSetup;
     // Transport parameters allows trans to be removed from CONNECTIONS
     private UniAddress address;
@@ -239,9 +239,10 @@ do {
             transport.send( request, response );
         }
     }
-int x = 0;
     void sessionSetup( ServerMessageBlock andx,
                             ServerMessageBlock andxResponse ) throws SmbException {
+        SmbException ex = null;
+
 synchronized( transport() ) {
         if( sessionSetup ) {
             return;
@@ -274,7 +275,14 @@ synchronized( transport() ) {
         }
 
         request.auth = auth;
-        transport.send( request, response );
+
+        try {
+            transport.send( request, response );
+        } catch (SmbAuthException sae) {
+            throw sae;
+        } catch (SmbException se) {
+            ex = se;
+        }
 
         if( response.isLoggedInAsGuest &&
                     "GUEST".equalsIgnoreCase( auth.username ) == false) {
@@ -288,6 +296,9 @@ synchronized( transport() ) {
             /* success - install the signing digest */
             transport.digest = request.digest;
         }
+
+        if (ex != null)
+            throw ex;
 }
     }
     void logoff( boolean inError ) {
