@@ -27,12 +27,15 @@ import jcifs.*;
 import jcifs.netbios.*;
 import jcifs.util.*;
 import jcifs.util.transport.*;
+import jcifs.dcerpc.*;
+import jcifs.dcerpc.msrpc.*;
 
 public class SmbTransport extends Transport implements SmbConstants {
 
     static final byte[] BUF = new byte[0xFFFF];
     static final SmbComNegotiate NEGOTIATE_REQUEST = new SmbComNegotiate();
     static LogStream log = LogStream.getInstance();
+    static HashMap dfsRoots = null;
 
     static synchronized SmbTransport getSmbTransport( UniAddress address, int port ) {
         return getSmbTransport( address, port, LADDR, LPORT );
@@ -160,7 +163,7 @@ public class SmbTransport extends Transport implements SmbConstants {
         try {
             connect( RESPONSE_TIMEOUT );
         } catch( IOException ioe ) {
-            throw new SmbException( "", ioe );
+            throw new SmbException( ioe.getMessage(), ioe );
         }
         return (capabilities & cap) == cap;
     }
@@ -179,6 +182,8 @@ public class SmbTransport extends Transport implements SmbConstants {
             } else {
                 socket = new Socket( address.getHostAddress(), 139, localAddr, localPort );
             }
+jcifs.Config.socketCount++;
+log.println(jcifs.Config.socketCount + ": new Socket: " + socket);
             socket.setSoTimeout( SO_TIMEOUT );
             out = socket.getOutputStream();
             in = socket.getInputStream();
@@ -187,7 +192,15 @@ public class SmbTransport extends Transport implements SmbConstants {
                     NbtAddress.getLocalName() );
             out.write( sbuf, 0, ssp.writeWireFormat( sbuf, 0 ));
             if (readn( in, sbuf, 0, 4 ) < 4) {
-                try { socket.close(); } catch(IOException ioe) {};
+                try {
+String s = socket.toString();
+                    socket.close();
+jcifs.Config.socketCount--;
+log.println(jcifs.Config.socketCount + ": socket closed: " + s);
+                } catch(IOException ioe) {
+log.println(jcifs.Config.socketCount + ": socket close exception:");
+ioe.printStackTrace(log);
+                }
                 throw new SmbException( "EOF during NetBIOS session request" );
             }
             switch( sbuf[0] & 0xFF ) {
@@ -200,7 +213,10 @@ public class SmbTransport extends Transport implements SmbConstants {
                     switch (errorCode) {
                         case NbtException.CALLED_NOT_PRESENT:
                         case NbtException.NOT_LISTENING_CALLED:
+String s = socket.toString();
                             socket.close();
+jcifs.Config.socketCount--;
+log.println(jcifs.Config.socketCount + ": socket closed: " + s);
                             break;
                         default:
                             disconnect( true );
@@ -243,6 +259,8 @@ public class SmbTransport extends Transport implements SmbConstants {
                 } else {
                     socket = new Socket( address.getHostAddress(), port, localAddr, localPort );
                 }
+jcifs.Config.socketCount++;
+log.println(jcifs.Config.socketCount + ": new Socket: " + socket);
                 socket.setSoTimeout( SO_TIMEOUT );
                 out = socket.getOutputStream();
                 in = socket.getInputStream();
@@ -286,7 +304,7 @@ public class SmbTransport extends Transport implements SmbConstants {
         try {
             super.connect( RESPONSE_TIMEOUT );
         } catch( TransportException te ) {
-            throw new SmbException( "", te );
+            throw new SmbException( te.getMessage(), te );
         }
     }
     protected void doConnect() throws IOException {
@@ -308,6 +326,9 @@ public class SmbTransport extends Transport implements SmbConstants {
         if( resp.dialectIndex > 10 ) {
             throw new SmbException( "This client does not support the negotiated dialect." );
         }
+        if (server.encryptionKeyLength != 8 && LM_COMPATIBILITY == 0) {
+            throw new SmbException("Encryption key length is not 8 as expected. This could indicate that the server requires NTLMv2. JCIFS does not fully support NTLMv2 but you can try setting jcifs.smb.lmCompatibility = 3.");
+        }
 
         /* Adjust negotiated values */
 
@@ -340,7 +361,10 @@ public class SmbTransport extends Transport implements SmbConstants {
         socket.shutdownOutput();
         out.close();
         in.close();
+String s = socket.toString();
         socket.close();
+jcifs.Config.socketCount--;
+log.println(jcifs.Config.socketCount + ": socket closed: " + s);
         digest = null;
     }
 
@@ -442,7 +466,10 @@ public class SmbTransport extends Transport implements SmbConstants {
             if (size < (HEADER_LENGTH + 1) || (4 + size) > rcv_buf_size ) {
                 throw new IOException( "Invalid payload size: " + size );
             }
-            if (resp.command == ServerMessageBlock.SMB_COM_READ_ANDX) {
+            int errorCode = Encdec.dec_uint32le( BUF, 9 ) & 0xFFFFFFFF;
+            if (resp.command == ServerMessageBlock.SMB_COM_READ_ANDX &&
+                        (errorCode == 0 ||
+                        errorCode == 0x80000005)) { // overflow indicator normal for pipe
                 SmbComReadAndXResponse r = (SmbComReadAndXResponse)resp;
                 int off = HEADER_LENGTH;
                                     /* WordCount thru dataOffset always 27 */
@@ -505,9 +532,10 @@ public class SmbTransport extends Transport implements SmbConstants {
                 if( req.auth == null ) {
                     throw new SmbException( resp.errorCode, null );
                 }
-                DfsReferral dr = getDfsReferral( req.auth, req.path );
-                referrals.add( dr );
-                throw dr;
+
+                DfsReferral[] drs = getDfsReferrals(req.auth, req.path, 1);
+                SmbFile.dfs.insert(req.path, drs[0]);
+                throw drs[0];
             case 0x80000005:  /* STATUS_BUFFER_OVERFLOW */
                 break; /* normal for DCERPC named pipes */
             default:
@@ -608,7 +636,7 @@ public class SmbTransport extends Transport implements SmbConstants {
         } catch( SmbException se ) {
             throw se;
         } catch( IOException ioe ) {
-            throw new SmbException( "", ioe );
+            throw new SmbException( ioe.getMessage(), ioe );
         }
 
         checkStatus( request, response );
@@ -619,61 +647,101 @@ public class SmbTransport extends Transport implements SmbConstants {
 
     /* DFS */
 
-    DfsReferral getDfsReferral( NtlmPasswordAuthentication auth,
-                String path ) throws SmbException {
-        String subpath, node, host;
-        DfsReferral dr = new DfsReferral();
-        int p, n, i, s;
-        UniAddress addr;
+    /* Split DFS path like \fs1.example.com\root5\link2\foo\bar.txt into at
+     * most 3 components (not including the first index which is always empty):
+     * result[0] = ""
+     * result[1] = "fs1.example.com"
+     * result[2] = "root5"
+     * result[3] = "link2\foo\bar.txt"
+     */
+    void dfsPathSplit(String path, String[] result)
+    {
+        int ri = 0, rlast = result.length - 1;
+        int i = 0, b = 0, len = path.length();
 
+        do {
+            if (ri == rlast) {
+                result[rlast] = path.substring(b);
+                return;
+            }
+            if (i == len || path.charAt(i) == '\\') {
+                result[ri++] = path.substring(b, i);
+                b = i + 1;
+            }
+        } while (i++ < len);
+
+        while (ri < result.length) {
+            result[ri++] = "";
+        }
+    }
+    DfsReferral[] getDfsReferrals(NtlmPasswordAuthentication auth,
+                String path,
+                int rn) throws SmbException {
         SmbTree ipc = getSmbSession( auth ).getSmbTree( "IPC$", null );
         Trans2GetDfsReferralResponse resp = new Trans2GetDfsReferralResponse();
         ipc.send( new Trans2GetDfsReferral( path ), resp );
 
-        subpath = path.substring( 0, resp.pathConsumed );
-        node = resp.referral.node;
-        if( subpath.charAt( 0 ) != '\\' ||
-                (i = subpath.indexOf( '\\', 1 )) < 2 ||
-                (p = subpath.indexOf( '\\', i + 1 )) < (i + 2) ||
-                node.charAt( 0 ) != '\\' ||
-                (s = node.indexOf( '\\', 1 )) < 2) {
-            throw new SmbException( "Invalid DFS path: " + path );
-        }
-        if ((n = node.indexOf( '\\', s + 1 )) == -1) {
-            n = node.length();
+        if (rn == 0 || resp.numReferrals < rn) {
+            rn = resp.numReferrals;
         }
 
-        dr.path = subpath.substring( p );
-        dr.node = node.substring( 0, n );
-        dr.nodepath = node.substring( n );
-        dr.server = node.substring( 1, s );
-        dr.share = node.substring( s + 1, n );
-        dr.resolveHashes = auth.hashesExternal;
+        DfsReferral[] drs = new DfsReferral[rn];
+        String[] arr = new String[4];
+        long expiration = System.currentTimeMillis() + Dfs.TTL * 1000;
+
+        for (int di = 0; di < drs.length; di++) {
+            DfsReferral dr = new DfsReferral();
                         /* NTLM HTTP Authentication must be re-negotiated
                          * with challenge from 'server' to access DFS vol. */
-        return dr;
-    }
-    DfsReferral lookupReferral( String unc ) {
-        synchronized( referrals ) {
-            DfsReferral dr;
-            ListIterator iter = referrals.listIterator();
-            int i, len;
-
-            while( iter.hasNext() ) {
-                dr = (DfsReferral)iter.next();
-                len = dr.path.length();
-                for( i = 0; i < len && i < unc.length(); i++ ) {
-                    if( dr.path.charAt( i ) != unc.charAt( i )) {
-                        break;
-                    }
-                }
-                if( i == len && (len == unc.length() || unc.charAt( len ) == '\\')) {
-                    return dr;
-                }
+            dr.resolveHashes = auth.hashesExternal;
+            dr.ttl = resp.referrals[di].ttl;
+            dr.expiration = expiration;
+            if (path.equals("")) {
+                dr.server = resp.referrals[di].path.substring(1).toLowerCase();
+            } else {
+                dfsPathSplit(resp.referrals[di].node, arr);
+                dr.server = arr[1];
+                dr.share = arr[2];
+                dr.path = arr[3];
             }
+            dr.pathConsumed = resp.pathConsumed;
+            drs[di] = dr;
         }
 
-        return null;
+        return drs;
+    }
+    FileEntry[] getDfsRoots(String domainName, NtlmPasswordAuthentication auth) throws IOException {
+        MsrpcDfsRootEnum rpc;
+        DcerpcHandle handle = null;
+
+        /* Procedure:
+         * Lookup a DC in the target domain
+         * Ask the DC for a referral for the domain (e.g. "\example.com")
+         * Do NetrDfsEnumEx on the server returned in the referral to
+         * get roots in target domain
+         */
+
+        UniAddress dc = UniAddress.getByName(domainName);
+        SmbTransport trans = SmbTransport.getSmbTransport(dc, 0);
+        DfsReferral[] dr = trans.getDfsReferrals(auth, "\\" + domainName, 1);
+
+        handle = DcerpcHandle.getHandle("ncacn_np:" +
+                    UniAddress.getByName(dr[0].server).getHostAddress() +
+                    "[\\PIPE\\netdfs]", auth);
+        try {
+            rpc = new MsrpcDfsRootEnum(domainName);
+            handle.sendrecv(rpc);
+            if (rpc.retval != 0)
+                throw new SmbException(rpc.retval, true);
+            return rpc.getEntries();
+        } finally {
+            try {
+                handle.close();
+            } catch(IOException ioe) {
+                if (log.level >= 4)
+                    ioe.printStackTrace(log);
+            }
+        }
     }
 }
 
